%this is a toy example of how to use the encoding method SD-VLAD, presented
%in the paper: "Efficient Human Action Recognition using Histograms of Motion Gradients
%and VLAD with Descriptor Shape Information", 
%Ionut C. Duta; Jasper R.R. Uijlings; Bogdan Ionescu; Kiyoharu Aizawa; 
%Alexander G. Hauptmann; Nicu Sebe. Multimedia Tools and Applications (MTAP), 2017.
%
%Please cite our work when using this code!


%In this example we use "fake" descriptors by generating them randomly. For
%using "real" video descriptors you can download the code for descriptor
%extraction from here: https://iduta.github.io/software.html

%generate the descriptors for creating the vocabulary
descCreateVocab=rand(5000, 72);

%create the vocabulary with the standard k-means, can be quite slow process
K=256; % the number of visual words
[idx, vocab]=kmeans(descCreateVocab, K); 

%compute the standard deviation for each cluster
st_d=zeros(size(vocab));
for i=1: size(vocab, 1)
    st_d(i, :)=std(descCreateVocab(idx==i, :));  
end

%generate the descriptors for which the encoding is performed
desc=rand(500, 72);

%make unit length of the vocabulary and descriptors for using inner
%product during the assignemnt step for computing the final SD-VLAD representation 
vocab_unitL=NormalizeRowsUnit(vocab);
desc_unitL=NormalizeRowsUnit(desc);

%encode the descriptors using SD-VLAD approach
SD_VLAD_encode=SD_VLAD_encoding(desc_unitL, vocab_unitL, st_d);